/*
 * Copyright (c) 1999-2001, SAP AG.  All Rights Reserved.
 */
package sap;

// -----( B2B Java IM Code Template v1.2
// -----( CREATED: Fri Oct 29 17:53:19 CEST 1999
// -----( ON-HOST: 155.56.73.48

import com.wm.data.*;
import com.wm.util.Values;
import com.wm.app.b2b.server.Service;
import com.wm.app.b2b.server.ServiceException;
// --- <<B2B-START-IMPORTS>> ---
import com.wm.pkg.sap.RfcCoder;
import com.wm.pkg.sap.bapi.BusinessDocumentCoder;
import pub.sap.transport.BAPI;
import wm.PartnerMgr.xtn.Transaction;
import java.io.*;
// --- <<B2B-END-IMPORTS>> ---

public final class demo
{
	// ---( internal utility methods )---

	final static demo _instance = new demo();

	static demo _newInstance() { return new demo(); }

	static demo _cast(Object o) { return (demo)o; }

	// ---( server methods )---



	public final static Values invoke_and_return_XML (Values in)
	{
		Values out = in;
		// --- <<B2B-START(invoke_and_return_XML)>> ---
		// [i] field:0:required serverName
		// [i] field:0:required $rfcname
		// [i] field:0:required $envelope {"XRFC","bXML"}
		// [o] field:0:required xmlData
		String serverName = in.getString("serverName"); 
		if (serverName != null)
			in.put("serverName", serverName);
		Values tmp = new Values();
		out=new Values();
		Values bla = new Values();
		bla.copyFrom(in);
		try
		{
		   tmp.copyFrom(Service.doInvoke("pub.sap.client", "invoke", Service.getSession(), bla));
		}
		catch(Exception e)
		{
		   tmp.copyFrom(Service.throwError(e));
		}
		tmp.put("$call", "false");
		tmp.put("repServerName", serverName);
		tmp.put("serverName", serverName);
		tmp.put("$rfcname", in.get("$rfcname"));
		String envelope = in.getString("$envelope");
		boolean bXML = (envelope!=null && envelope.equals("bXML"));
		RfcCoder coder=new RfcCoder(bXML);
		try
		{
			byte[] result = coder.encodeToBytes(tmp);
			out.remove("$error");
			out.remove("$errorType");
			if (coder.getIsoEncoding()!=null) //if an encoding was recognized, use it!
				out.put("xmlData", new String(result, coder.getIsoEncoding()));
			else
				out.put("xmlData", new String(result, "UTF8"));
		}
		catch(Exception e)
		{
			out=Service.throwError(e);
		}
		// --- <<B2B-END>> ---
		return out;
	}


	public final static Values writeSAPXMLFile (Values in)
	{
		Values out = in;
		// --- <<B2B-START(writeSAPXMLFile)>> ---
		// This service will convert RFCs or IDocs coming from an R/3 system
		// to SAP-XML (IDoc-XML or RFC-XML) and write the result to an output
		// file in the 'sap/pub' directory. To call it create a routing rule,
		// choose the B2B transport and specify this service of call it directly.
		
		try
		{
		 
		// If you want to know what keys are passed in with the pipeline
		// uncomment the next lines
		//	String[] keys = in.getValueKeys();
		//	for (int i=0; i<keys.length; i++) 
		//		System.out.println(keys[i] + " = " + in.get(keys[i]));
		
			String fileName = "writeSAPXMLFile";
			String fNextension = ".xml";
		
			// check whether we got XML right away
			if (in.containsKey("xmlData")) 
			{
				System.out.println("Got XML input, nothing left to do ;-)");
				if (in.containsKey("$tid"))
					fileName = out.getString("$tid");
				else if (in.containsKey("$rfcname"))
					fileName = out.getString("$rfcname");
				write2File( out, fileName, fNextension);
			}
			// check for calls coming from R/3 of via B2B transport
			else if (in.containsKey("IDOC_DATA") || in.containsKey("IDOC_DATA_REC_40"))
			{
				System.out.println("It's an IDOC coming from R/3 or via B2B transport!");
				out.copyFrom( Service.doInvoke("pub.sap.idoc", "encode", in) );
				fileName = in.getString("$tid");
				write2File( out, fileName, fNextension);
				//clean up pipeline
				out.remove("IDOC_DATA");
				out.remove("IDOC_DATA_REC_40");
				out.remove("IDOC_CONTROL");
				out.remove("IDOC_CONTROL_REC_40");
				out.remove("boundNode");
			}
			else if (in.containsKey("$rfcname"))
			{
				System.out.println("It's an RFC coming from R/3 or via B2B transport!");
				out.copyFrom( Service.doInvoke("pub.sap.rfc", "encode", in) );
				if (in.containsKey("$tid"))
					fileName = out.getString("$tid");
				else if (in.containsKey("$rfcname"))
					fileName = out.getString("$rfcname");
				write2File( out, fileName, fNextension);
			}
			else
				System.out.println("Unknown content type, input needs to come from SAP System via RFC or IDoc.");
		}
		catch (Exception e )
		{
			System.out.println(e.toString() );
		}
		
		// --- <<B2B-END>> ---
		return out;
	}

	public static final void invokeBAPIReturningbXML (IData pipeline)
		throws ServiceException
	{
		// --- <<B2B-START(invokeBAPIReturningbXML)>> ---
		// @sigtype java 3.5
		// [i] field:0:optional serverName
		// [i] field:0:optional mode {"sync", "async, "routing"}
		// [o] field:0:required xmlData
		IDataCursor idc=pipeline.getCursor();
		byte mode=SYNC_MODE;
		//get selected mode, if none is specified, set to sync --> RFC call
		if (idc.first("mode"))
		{
			String modeStr=(String)idc.getValue();
			if(modeStr!=null)
			{
				if(modeStr.equals(SYNC))
					mode = SYNC_MODE;
				else if(modeStr.equals(ASYNC))
					mode = ASYNC_MODE;
				else if(modeStr.equals(ROUTE))
					mode = ROUTING_MODE;
			}
		}
		String serverName=null;
		if (idc.first("serverName"))
			serverName=(String)idc.getValue();
		if(serverName==null && mode!=ROUTING_MODE)
			throw new ServiceException("Missing required parameter 'serverName'");
		try
		{
			BusinessDocumentCoder coder = new BusinessDocumentCoder();
			Values request=new Values();
			request.copyFrom(Values.use(pipeline));
			request.put(BusinessDocumentCoder.CALL_INDICATOR_KEY, "true");
			byte[] data=null;
			Values response = null;
			Values transParams = new Values(2);
			transParams.put("server", serverName);
			try
			{
				switch(mode)
				{
					case ASYNC_MODE:
						transParams.put("mode", BAPI.ASYNC_MODE);
						//if async is wished, always provide a TID
						if(!request.containsKey(Transaction.TID))
						{
							Values tv  = new Values(1);
							tv.put("serverName", serverName);
						   	tv = Service.doInvoke("pub.sap.client", "createTID", tv);
	        				String tid = tv.getString(Transaction.TID);		
							request.put(Transaction.TID, tid);
						}
						request.put("transportParams", transParams);
						response = Service.doInvoke("pub.sap.transport.BAPI","OutboundProcess", request);
						break;
					case SYNC_MODE:
						transParams.put("mode", BAPI.SYNC_MODE);
						request.remove(Transaction.TID); //ensure there's no transaction assigned to sync call
						request.put("transportParams", transParams);
						response = Service.doInvoke("pub.sap.transport.BAPI","OutboundProcess", request);
						break;
					case ROUTING_MODE:
						response = Service.doInvoke("pub.sap.transport.BAPI","InboundProcess", request);
						break;
				}			
				data = coder.encodeToBytes(response);
			}
			catch(Exception e)
			{	
				data = coder.encodeToBytes(request.copyFrom(Service.throwError(e)));
			}
			
			String xmlOut;
			if (coder.getIsoEncoding()!=null) //if an encoding was recognized, use it!
				xmlOut=new String(data, coder.getIsoEncoding());
			else
				xmlOut=new String(data, "UTF8");
			idc.insertAfter("xmlData", xmlOut);
		}
		catch(Exception e)
		{
			throw new ServiceException(e);
		}
		idc.destroy();
		// --- <<B2B-END>> ---
	}

	// --- <<B2B-START-SHARED>> ---
	private static final String SYNC = "sync";
	private static final String ASYNC = "async";
	private static final String ROUTE = "routing";
	
	private static final byte SYNC_MODE = 0;
	private static final byte ASYNC_MODE = 1;
	private static final byte ROUTING_MODE = 2;
	public static final boolean write2File(Values out, String fileName, String extension) 
		throws java.io.IOException 
	{
		String filePath = "packages" + File.separator + Service.getPackageName(out) +
				File.separator + "pub" + File.separator + fileName + extension;		
		FileWriter fout = new FileWriter(new File(filePath));
		fout.write( (String) out.get("xmlData"));
		fout.close();
		return (true);
	
	}
	// --- <<B2B-END-SHARED>> ---
}

