package sap.demo.idoc;

// -----( B2B Java IM Code Template v1.2
// -----( CREATED: Tue Sep 07 18:39:09 CEST 1999
// -----( ON-HOST: P33078

import com.wm.util.Values;
import com.wm.app.b2b.server.Service;
// --- <<B2B-START-IMPORTS>> ---
import com.wm.pkg.sap.idoc.*;
import com.wm.util.Table;
import java.util.Vector;
// --- <<B2B-END-IMPORTS>> ---

public final class mappings
{
	// ---( internal utility methods )---

	final static mappings _instance = new mappings();

	static mappings _newInstance() { return new mappings(); }

	static mappings _cast(Object o) { return (mappings)o; }

	// ---( server methods )---



	public final static Values orders (Values in)
	{
		Values out = in;
		// --- <<B2B-START(orders)>> ---
		// @specification pub.sap.idoc.routing:inbound

	   /*
			Mapping algorithm:

			Sender:     If (E1EDKA1/PARVW) == "AG"
						   then sender = E1EDKA1/PARTN

			Receiver:   If (E1EDKA1/PARVW) == "LF"
						   then receiver = E1EDKA1/PARTN

			MsgType:    EDI_DC/MESTYP

			Set fields: EDI_DC/SNDPRN = sender
						EDI_DC/SNDPRT = "KU"
		*/

		String msgType = in.getString("msgType");
		if (msgType == null)
			return Service.throwError("Missing parameter: 'msgType'");
		else if (!msgType.equals("ORDERS"))
			return Service.throwError("IDOC is not of type 'ORDERS'");
		try
		{
			IDOC idoc = new IDOC(in);

			IDOCSegment segment = idoc.findSegment("E1EDKA1", "PARVW", "AG");
			String sender = null;
			if (segment!=null)
				sender = segment.getSDATA("PARTN");

			segment = idoc.findSegment("E1EDKA1", "PARVW", "LF");
			String receiver = null;
			if (segment!=null)
				receiver = segment.getSDATA("PARTN");

			IDOCControl control = idoc.getControlRecord();
			if (sender!=null)
			{
				control.setField(IDOCControl.SNDPRN, sender);
				control.setField(IDOCControl.SNDPRT, "KU");
				out.put("sender", sender);
			}
			if (receiver!=null)
				out.put("receiver", receiver);
			idoc.commitChanges();
		}
		catch (Exception e)
		{
			out.copyFrom(Service.throwError(e));
		}

		// --- <<B2B-END>> ---
		return out;
	}


	public final static Values ordrsp (Values in)
	{
		Values out = in;
		// --- <<B2B-START(ordrsp)>> ---
		// @specification pub.sap.idoc.routing:outbound

		/*
			Mapping algorithm:

			Sender:    If (E1EDKA1/PARVW) == "AG" 
						   then sender = E1EDKA1/LIFNR

			Receiver:  Not modified.
			MsgType:   Not modified.

			Set fields: EDI_DC/SNDPRN = sender
						EDI_DC/SNDPRT = "LI"
		*/

		String msgType = in.getString("msgType");
		if (msgType == null)
			return Service.throwError("Missing parameter: 'msgType'");
		else if (!msgType.equals("ORDRSP"))
			return Service.throwError("IDOC is not of type 'ORDRSP'");

		try
		{
			IDOC idoc = new IDOC(in);
			IDOCSegment segment = idoc.findSegment("E1EDKA1", "PARVW", "AG");
			String sender = null;
			if (segment!=null)
				sender = segment.getSDATA("LIFNR");
			IDOCControl control = idoc.getControlRecord();
			if (sender!= null)
			{
				control.setField(IDOCControl.SNDPRN, sender);
				control.setField(IDOCControl.SNDPRT, "LI");
			}
			idoc.commitChanges();
		}
		catch (Exception e)
		{
			out.copyFrom(Service.throwError(e));
		}
		// --- <<B2B-END>> ---
		return out;
	}
}

